from pyomo.environ import *
import matplotlib.pyplot as plt
from sympy import N

def cargarModelo():
    # max f1 = X1 <br>
    # max f2 = 3 X1 + 4 X2 <br>
    # st  X1 <= 20 <br>
    #     X2 <= 40 <br>
    #     5 X1 + 4 X2 <= 200 <br>

    model = ConcreteModel()

    model.X1 = Var(within=NonNegativeReals)
    model.X2 = Var(within=NonNegativeReals)

    model.C1 = Constraint(expr = model.X1 <= 20)
    model.C2 = Constraint(expr = model.X2 <= 40)
    model.C3 = Constraint(expr = 5 * model.X1 + 4 * model.X2 <= 200)

    model.f1 = Var()
    model.f2 = Var()
    model.C_f1 = Constraint(expr= model.f1 == model.X1)
    model.C_f2 = Constraint(expr= model.f2 == 3 * model.X1 + 4 * model.X2)
    model.O_f1 = Objective(expr= model.f1  , sense=maximize)
    model.O_f2 = Objective(expr= model.f2  , sense=maximize)

    return model

def encontrarExtremos(model):

    model.O_f2.deactivate()
    model.O_f1.activate()

    solver = SolverFactory('glpk')
    solver.options['tmlim'] = 5
    solver.options['mipgap'] = 0.0
    solver.solve(model, tee=True )

    print('Extremo 1')
    print( '( X1 , X2 ) = ( ' + str(value(model.X1)) + ' , ' + str(value(model.X2)) + ' )')
    print( 'f1 = ' + str(value(model.f1)) )
    print( 'f2 = ' + str(value(model.f2)) )
    f2_min = value(model.f2)
    f1_max = value(model.f1)

    # ## max f2

    model.O_f2.activate()
    model.O_f1.deactivate()

    #solver = SolverFactory('cplex')
    solver = SolverFactory('glpk')
    solver.solve(model);

    print('Extremo 2')
    print( '( X1 , X2 ) = ( ' + str(value(model.X1)) + ' , ' + str(value(model.X2)) + ' )')
    print( 'f1 = ' + str(value(model.f1)) )
    print( 'f2 = ' + str(value(model.f2)) )
    f2_max = value(model.f2)
    f1_min = value(model.f1)

    model.O_f1.activate()

    return f2_min, f2_max, f1_min, f1_max, solver

def main():

    model = cargarModelo()
    f2_min, f2_max, f1_min, f1_max, solver = encontrarExtremos(model)

if __name__ == "__main__":

    main()